# nGPT as a CLI Framework

This guide explains how to leverage nGPT's CLI components to build your own command-line applications. nGPT provides a rich set of reusable CLI utilities that can accelerate the development of AI-powered command-line tools.

## Overview

nGPT's `cli.py` module contains several reusable components that you can incorporate into your own CLI applications:

- **Interactive Chat Interface**: A fully-featured chat UI with history management
- **Markdown Rendering**: Beautiful formatting for markdown with syntax highlighting
- **Real-time Streaming**: Tools for handling streaming content with live updates
- **CLI Configuration System**: Robust configuration management
- **Terminal Utilities**: Helpers for colorized output and terminal formatting

## Getting Started

To use nGPT as a CLI framework, first install it:

```bash
pip install ngpt
```

This will install nGPT with all required dependencies, including:
- `requests` for API communication
- `rich` for markdown rendering and syntax highlighting
- `prompt_toolkit` for interactive features

## Available Components

### Interactive Chat Session

The `interactive_chat_session` function provides a complete interactive chat experience:

```python
from ngpt import NGPTClient, load_config
from ngpt.cli import interactive_chat_session

# Initialize client
config = load_config()
client = NGPTClient(**config)

# Start interactive session
interactive_chat_session(
    client=client,
    web_search=False,
    no_stream=False,
    temperature=0.7,
    top_p=1.0,
    max_tokens=None,
    log_file=None,
    preprompt=None,
    prettify=False,
    renderer='auto',
    stream_prettify=False
)
```

This function handles:
- User input with prompt toolkit
- Displaying assistant responses
- Conversation history management
- Special commands like `/clear`, `/help`, etc.

#### Error Handling

The interactive chat session includes built-in error handling, but you should wrap the initialization of the client in a try/except block:

```python
try:
    config = load_config()
    client = NGPTClient(**config)
    interactive_chat_session(client=client)
except Exception as e:
    print(f"Error setting up chat session: {e}", file=sys.stderr)
    sys.exit(1)
```

### Markdown Rendering

nGPT provides utilities for rendering markdown with syntax highlighting:

```python
from ngpt.cli import prettify_markdown, has_markdown_renderer

# Check if renderer is available
if has_markdown_renderer(renderer='rich'):
    # Render markdown
    formatted_text = prettify_markdown(
        "# Hello World\n```python\nprint('Hello')\n```",
        renderer='rich'
    )
    print(formatted_text)
```

Available renderers include:
- `rich`: Uses the Rich library for colorful terminal output
- `glow`: Uses the Glow CLI tool if installed

You can check available renderers:

```python
from ngpt.cli import show_available_renderers
show_available_renderers()
```

#### Fallback Behavior

The markdown rendering system has built-in fallback logic:

```python
# Auto-select the best available renderer
formatted = prettify_markdown(markdown_text)

# Explicit fallback handling
if has_markdown_renderer(renderer='rich'):
    formatted = prettify_markdown(markdown_text, renderer='rich')
elif has_markdown_renderer(renderer='glow'):
    formatted = prettify_markdown(markdown_text, renderer='glow')
else:
    # No fancy renderer available, just print plain text
    formatted = markdown_text
```

### Real-time Markdown Streaming

For real-time rendering of streaming content, the `prettify_streaming_markdown` function returns an object with an `update_content` method:

```python
from ngpt.cli import prettify_streaming_markdown
from ngpt import NGPTClient, load_config

client = NGPTClient(**load_config())

# Create a streamer object
streamer = prettify_streaming_markdown(
    renderer='rich',
    is_interactive=False,
    header_text="AI Response:"
)

# Use the streamer's update_content method with the client
full_response = ""
for chunk in client.chat("Explain quantum computing with code examples", stream=True):
    full_response += chunk
    # Call the update_content method on the streamer object
    streamer.update_content(full_response)
```

This creates a live-updating display that refreshes as new content arrives. The function returns a class instance with an `update_content` method, not the method itself.

### CLI Configuration Management

nGPT provides tools for managing CLI configurations:

```python
from ngpt.cli import handle_cli_config, show_cli_config_help

# Show help
show_cli_config_help()

# Get a configuration value
value = handle_cli_config('get', 'temperature')

# Set a configuration value
handle_cli_config('set', 'temperature', '0.8')

# List all configuration options
options = handle_cli_config('list')
```

#### Error Handling for Configuration

```python
try:
    value = handle_cli_config('get', 'temperature')
    if value is None:
        # Option not set, use default
        value = 0.7
except Exception as e:
    print(f"Error accessing configuration: {e}", file=sys.stderr)
    # Use fallback value
    value = 0.7
```

### Terminal Formatting Utilities

Colorize your CLI output with terminal utilities:

```python
from ngpt.cli import ColoredHelpFormatter, supports_ansi_colors
import argparse

# Check if terminal supports colors
if supports_ansi_colors():
    # Use colored output
    print("\033[1;32mSuccess!\033[0m")
else:
    # Fallback to plain text
    print("Success!")

# Create a parser with colorized help
parser = argparse.ArgumentParser(
    description="My custom CLI application",
    formatter_class=ColoredHelpFormatter
)

# Add arguments
parser.add_argument("--option", help="This help text will be formatted with colors")
```

The `ColoredHelpFormatter` automatically adapts to the terminal's capabilities and will fall back to standard formatting if colors aren't supported.

## Complex Example: Building a Custom AI CLI

Here's a complete example of building a specialized AI CLI tool using nGPT components:

```python
#!/usr/bin/env python
import argparse
import sys
from pathlib import Path
from ngpt import NGPTClient, load_config
from ngpt.cli import (
    ColoredHelpFormatter,
    interactive_chat_session,
    prettify_markdown,
    prettify_streaming_markdown,
    has_markdown_renderer
)

def main():
    # Set up argument parser with colorized help
    parser = argparse.ArgumentParser(
        description="CodeAI - Generate and explain code using AI",
        formatter_class=ColoredHelpFormatter
    )
    
    # Add arguments
    parser.add_argument("prompt", nargs="?", help="Code description or question")
    parser.add_argument("-l", "--language", default="python", help="Programming language")
    parser.add_argument("-i", "--interactive", action="store_true", help="Start interactive mode")
    parser.add_argument("-e", "--explain", action="store_true", help="Explain existing code")
    parser.add_argument("-f", "--file", type=str, help="Read from or write to file")
    parser.add_argument("-p", "--prettify", action="store_true", help="Prettify output")
    
    args = parser.parse_args()
    
    # Initialize client
    try:
        config = load_config()
        client = NGPTClient(**config)
    except Exception as e:
        print(f"Error initializing AI client: {e}", file=sys.stderr)
        sys.exit(1)
    
    # Interactive mode
    if args.interactive:
        print("CodeAI Interactive Mode - Type /help for commands")
        preprompt = f"You are an expert {args.language} developer. Provide concise, clear code examples."
        interactive_chat_session(
            client=client,
            preprompt=preprompt,
            prettify=args.prettify,
            renderer='rich' if has_markdown_renderer(renderer='rich') else 'auto'
        )
        return
    
    # Explain mode - read code from file
    if args.explain and args.file:
        try:
            with open(args.file, 'r') as f:
                code = f.read()
            
            prompt = f"Explain this {args.language} code:\n\n{code}"
            
            if args.prettify and has_markdown_renderer():
                print("Analyzing code...")
                streamer = prettify_streaming_markdown(renderer='rich')
                full_response = ""
                for chunk in client.chat(prompt, stream=True):
                    full_response += chunk
                    streamer.update_content(full_response)
            else:
                print("Analyzing code...")
                response = client.chat(prompt)
                print("\nExplanation:")
                print(response)
        except Exception as e:
            print(f"Error: {e}", file=sys.stderr)
            sys.exit(1)
        return
    
    # Generate code mode
    if args.prompt:
        preprompt = f"You are an expert {args.language} developer. Generate clean, well-documented {args.language} code."
        
        if args.file:
            # Generate code and save to file
            print(f"Generating {args.language} code...")
            code = client.generate_code(
                args.prompt,
                language=args.language
            )
            
            try:
                with open(args.file, 'w') as f:
                    f.write(code)
                print(f"Code written to {args.file}")
            except Exception as e:
                print(f"Error writing to file: {e}", file=sys.stderr)
                sys.exit(1)
        else:
            # Generate and display code
            print(f"Generating {args.language} code...\n")
            
            if args.prettify and has_markdown_renderer():
                markdown = f"```{args.language}\n"
                
                # Stream and capture code
                code = ""
                for chunk in client.generate_code(
                    args.prompt,
                    language=args.language,
                    stream=True
                ):
                    code += chunk
                
                markdown += code + "\n```"
                print(prettify_markdown(markdown))
            else:
                # Simple streaming output
                for chunk in client.generate_code(
                    args.prompt,
                    language=args.language,
                    stream=True
                ):
                    print(chunk, end="", flush=True)
                print()  # Final newline
    else:
        parser.print_help()

if __name__ == "__main__":
    main()
```

## Working with Different Renderer Options

nGPT supports multiple markdown renderers:

### Rich Renderer

Using the Rich library for terminal output:

```python
from ngpt.cli import prettify_markdown

# Rich renderer with custom styling
formatted = prettify_markdown(markdown_text, renderer='rich')
print(formatted)
```

### Glow Renderer

If you have the Glow CLI tool installed:

```python
from ngpt.cli import prettify_markdown, has_glow_installed

if has_glow_installed():
    formatted = prettify_markdown(markdown_text, renderer='glow')
    print(formatted)
```

### Auto Selection

Let nGPT choose the best available renderer:

```python
formatted = prettify_markdown(markdown_text, renderer='auto')
print(formatted)
```

## Detecting Terminal Capabilities

nGPT provides utilities to detect terminal capabilities:

```python
from ngpt.cli import supports_ansi_colors

if supports_ansi_colors():
    # Use colored output
    print("\033[1;32mSuccess!\033[0m")
else:
    # Fallback to plain text
    print("Success!")
```

## Text Input Components

nGPT includes utilities for handling user input:

### Prompt Input

For simple single-line input:

```python
from prompt_toolkit import prompt

user_input = prompt("Enter your query: ")
```

### Multiline Input

When nGPT is installed with full dependencies, it provides components for rich multiline editing in the interactive session. These components are used internally by the `interactive_chat_session` function.

## Advanced Usage: Creating a Web Search Tool

This example creates a specialized web search tool using nGPT components:

```python
#!/usr/bin/env python
import argparse
import sys
from ngpt import NGPTClient, load_config
from ngpt.cli import (
    ColoredHelpFormatter,
    prettify_streaming_markdown,
    has_markdown_renderer
)

def main():
    # Set up colorized argument parser
    parser = argparse.ArgumentParser(
        description="AI-powered web search tool",
        formatter_class=ColoredHelpFormatter
    )
    
    parser.add_argument("query", nargs="?", help="Search query")
    parser.add_argument("-p", "--prettify", action="store_true", help="Prettify output")
    
    args = parser.parse_args()
    
    if not args.query:
        parser.print_help()
        return
    
    # Initialize client
    try:
        config = load_config()
        client = NGPTClient(**config)
    except Exception as e:
        print(f"Error: {e}", file=sys.stderr)
        sys.exit(1)
    
    # System prompt that encourages citations
    system_prompt = """You are a web research assistant. 
    For each query, search the internet and provide:
    1. A concise summary of the most important information
    2. Key facts with references to their sources
    3. Direct quotes should be in blockquotes
    Use markdown formatting for better readability."""
    
    print(f"Searching for: {args.query}\n")
    
    # Create messages with system prompt
    messages = [
        {"role": "system", "content": system_prompt},
        {"role": "user", "content": args.query}
    ]
    
    if args.prettify and has_markdown_renderer():
        # Stream with real-time markdown rendering
        streamer = prettify_streaming_markdown(
            renderer='rich',
            header_text="Search Results"
        )
        
        full_response = ""
        for chunk in client.chat(
            "",
            messages=messages,
            web_search=True,
            stream=True
        ):
            full_response += chunk
            streamer.update_content(full_response)
    else:
        # Regular streaming output
        print("Results:")
        for chunk in client.chat(
            "",
            messages=messages,
            web_search=True,
            stream=True
        ):
            print(chunk, end="", flush=True)
        print()  # Final newline

if __name__ == "__main__":
    main()
```

## Logging and Debugging

nGPT CLI components work with Python's standard logging module. You can configure logging for better debugging:

```python
import logging

# Set up logging
logging.basicConfig(
    level=logging.INFO,
    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
)

# Use nGPT components with logging available
from ngpt.cli import prettify_markdown
try:
    result = prettify_markdown(text)
except Exception as e:
    logging.error(f"Failed to render markdown: {e}")
    result = text  # Fallback to plain text
```

## Conclusion

nGPT's CLI components provide a rich toolkit for building your own AI-powered command-line applications. By leveraging these pre-built components, you can rapidly develop sophisticated CLI tools with features like:

- Interactive chat interfaces
- Beautiful markdown rendering
- Real-time streaming with live updates
- Configuration management
- Colorized terminal output

For more examples, see the [CLI Component Examples](../examples/cli_components.md) section. 